
package com.yeejoin.amos.bank.service.impl;

import com.google.common.collect.Maps;
import com.yeejoin.amos.bank.common.enums.AbnormalStateEnum;
import com.yeejoin.amos.bank.dao.entity.AlarmPoint;
import com.yeejoin.amos.bank.dao.repository.IAlarmPointRepository;
import com.yeejoin.amos.bank.remote.RemoteAlarmTiggerService;
import com.yeejoin.amos.bank.service.IAlarmPointService;
import com.yeejoin.amos.op.core.jpa.GenericManagerImpl;
import com.yeejoin.amos.spc.business.dao.mapper.InputItemMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class AlarmServiceImpl extends GenericManagerImpl<AlarmPoint, String>  implements IAlarmPointService {


	@Autowired
	private IAlarmPointRepository iAlarmPointDao;

	@Autowired
	private InputItemMapper inputMapper;

	@Autowired
	private RemoteAlarmTiggerService alarmTiggerService;

	public AlarmServiceImpl(IAlarmPointRepository repository) {
		super(repository);
		this.iAlarmPointDao = repository;
	}


	@Override
	public void saveItem(AlarmPoint alarmPoint) {
		iAlarmPointDao.save(alarmPoint);
	}

	@Override
	public void deleteItem(List<AlarmPoint> alarmPoints) {
		iAlarmPointDao.deleteAll(alarmPoints);
	}

	@Override
	public void deleteItems(List<AlarmPoint> alarmPoints) {
		List<Long> ids = new ArrayList<>();
		if(null != alarmPoints && alarmPoints.size() > 0 ) {
			alarmPoints.forEach(alarmPoint -> {
				ids.add(alarmPoint.getId());
			});
//			iAlarmPointDao.deleteByIdIn(ids);
			inputMapper.deletePointAlarmByIds(ids);
		}

    }

	@Override
	public List<AlarmPoint> selectPointTypeAndPointId(String type, Long pointId) {

		return iAlarmPointDao.selectPointTypeAndPointId( type,  pointId);

	}

	@Override
	public List<AlarmPoint> selectPointTypeAndPointId(String type, Long pointId,Long cpblId) {

		return iAlarmPointDao.selectPointTypeAndPointId( type,pointId,cpblId);

	}

	@Override
	public void alarmTigger(String xmlData,String token) {
		Map<String, String> params =  Maps.newHashMap();
		params.put("xmlData", xmlData);
		String alarmTiggerGet = alarmTiggerService.sendAlarm(xmlData);
		//String alarmTiggerGet = alarmTiggerService.alarmTiggerGet(DeviceUriEnum.触发清除告警, token, params);
	}


	@Override
	public List<AlarmPoint> queryAutoInspection(String pointId,String content) {
		return iAlarmPointDao.selectPointTypeAndPointId(pointId,content);
	}

	@Override
	public Map<String,Object> getSevenDayscount() throws ParseException {
		Map<String,Object> result = new HashMap<>();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		Calendar c = Calendar.getInstance();
		c.add(Calendar.DATE, -6);
		//紧急告警
		List<Long>  urgent = inputMapper.getSevenDayscount(AbnormalStateEnum.E.getName());
		result.put("urgent",urgent);
		//重要告警
		List<Long>  important = inputMapper.getSevenDayscount(AbnormalStateEnum.M.getName());
		result.put("important",important);
		//次要告警
		List<Long>  secondary = inputMapper.getSevenDayscount(AbnormalStateEnum.S.getName());
		result.put("secondary",secondary);
		//警告告警
		List<Long>  warning = inputMapper.getSevenDayscount(AbnormalStateEnum.W.getName());
		result.put("warning",warning);
		List<String> xAxis = new ArrayList<>();

		for (int i = 0; i < 7; i++) {
			xAxis.add(sdf.format(c.getTime()).substring(5,sdf.format(c.getTime()).length()));
			c.add(Calendar.DATE, 1);
		}
		//时间轴
		result.put("xAxis",xAxis);
		return result;
	}
}
