package com.yeejoin.amos.bank.webSocket;

import static com.yeejoin.amos.bank.constants.AlarmConstant.ALARM_ABNORMAL_STATUS;
import static com.yeejoin.amos.bank.constants.AlarmConstant.ALARM_NORMAL_STATUS;
import static com.yeejoin.amos.bank.constants.AlarmConstant.CLEAR_PERSON;
import static com.yeejoin.amos.bank.constants.AlarmConstant.ALARM_DEVICE_TYPE;
import static com.yeejoin.amos.bank.constants.AlarmConstant.ALARM_DEVICE_MAJOR;


import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import com.yeejoin.amos.bank.TopographyNodeDetail;
import com.yeejoin.amos.bank.dao.repository.ITopographyNodeDetailRepository;
import com.yeejoin.amos.bank.dao.repository.ITopographyNodeRepository;
import org.java_websocket.WebSocket;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ServerHandshake;
import org.apache.log4j.Logger;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.support.PropertiesLoaderUtils;
import org.springframework.data.domain.Sort;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.bank.common.enums.AlarmPointEnum;
import com.yeejoin.amos.bank.common.enums.AlarmPointTypeEnum;
import com.yeejoin.amos.bank.dao.entity.Alarm;
import com.yeejoin.amos.bank.dao.entity.AlarmPoint;
import com.yeejoin.amos.bank.dao.mapper.AlarmMapper;
import com.yeejoin.amos.bank.dao.repository.IAlarmRepository;
import com.yeejoin.amos.bank.remote.client.PatrolRemoteClient;
import com.yeejoin.amos.bank.service.IAlarmPointService;
import com.yeejoin.amos.bank.service.IAlarmService;
import com.yeejoin.amos.bank.service.ITopographyService;
import com.yeejoin.amos.bank.utils.DateTimeKit;
import com.yeejoin.amos.bank.utils.HttpUtil;
import com.yeejoin.amos.bank.utils.JsonUtil;
import com.yeejoin.amos.op.core.common.response.CommonResponse;
import com.yeejoin.amos.spc.exception.YeeException;

import io.micrometer.core.instrument.util.StringUtils;
import org.springframework.util.ObjectUtils;

/**
 * 湿度动环告警
 *
 * @author DELL
 */
public class ShiduAlarmWebSocketClient extends WebSocketClient {

    private static final Logger log = Logger.getLogger(ShiduAlarmWebSocketClient.class);
    private static final String TAB = "\r\n";
    private static ApplicationContext allpicationContext;
    private ITopographyNodeRepository iTopographyNodeRepository;
    private ITopographyNodeDetailRepository iTopographyNodeDetailDao;
    private IAlarmPointService alarmPointService;
    private IAlarmService alarmService;
    private ITopographyService iTopographyService;
    private AmosWsClient client;
	private IAlarmRepository iAlarmDao;
    private String address;
    private PatrolRemoteClient patrolRemoteClient;

    private AlarmMapper alarmMapper;

    public ShiduAlarmWebSocketClient(String url) throws URISyntaxException {
        super(new URI(url));
    }

    public static void setAllpicationContext(ApplicationContext context) {
        allpicationContext = context;
    }

    private static String getAuthOrg(String authOrg) {
         if (!StringUtils.isEmpty(authOrg) && authOrg.startsWith("[") && authOrg.endsWith("]")) {
         	String str = authOrg.substring(1, authOrg.length() - 1).replace(" ", "").replace(",", "#");
            return "#"+ str;
 		 }
         return null;
    }

    public static synchronized void synMonitorAlarmPoint() {
        System.out.println("-------------------shidu Monitoring of ring alarms start-------------------------");
        try {
            Properties props = PropertiesLoaderUtils.loadAllProperties("application.properties");
            String monitorAlarmWebSocketClientUrl = (String) props.get("params.remoteWebsocketUrl") + "metricData?subData=shiduAlarm";
            ShiduAlarmWebSocketClient client = new ShiduAlarmWebSocketClient(monitorAlarmWebSocketClientUrl);
            client.connect();
            while (!client.getReadyState().equals(WebSocket.READYSTATE.OPEN)) {
                log.info("connecting... shidu Dynamic ring alarm----" + client.getReadyState());
            }
            System.out.println("build " + monitorAlarmWebSocketClientUrl + "connected...");
        } catch (URISyntaxException | IOException e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
    }

    @Override
    public void onOpen(ServerHandshake shake) {
        log.info("MonitorAlarmWebSocketClient Shake hands...");
        for (Iterator<String> it = shake.iterateHttpFields(); it.hasNext(); ) {
            String key = it.next();
            log.info(key + ":" + shake.getFieldValue(key));
        }
        alarmPointService = (IAlarmPointService) allpicationContext.getBean(IAlarmPointService.class);
        iTopographyService = (ITopographyService) allpicationContext.getBean(ITopographyService.class);
        client = (AmosWsClient) allpicationContext.getBean(AmosWsClient.class);
        alarmService = (IAlarmService) allpicationContext.getBean(IAlarmService.class);
        patrolRemoteClient= (PatrolRemoteClient) allpicationContext.getBean(PatrolRemoteClient.class);
        iAlarmDao =(IAlarmRepository) allpicationContext.getBean(IAlarmRepository.class);
        alarmMapper = (AlarmMapper) allpicationContext.getBean(AlarmMapper.class);
        iTopographyNodeRepository = (ITopographyNodeRepository) allpicationContext.getBean(ITopographyNodeRepository.class);
        iTopographyNodeDetailDao = (ITopographyNodeDetailRepository) allpicationContext.getBean(ITopographyNodeDetailRepository.class);
		try {
			Properties props = PropertiesLoaderUtils.loadAllProperties("application.properties");
			address = (String) props.get("params.patrol");
		} catch (IOException e) {
			e.printStackTrace();
		}
    }

    @Override
    @Transactional
    public void onMessage(String paramString) {
        try {
            log.info("Received  shidu message：" + paramString);
            String formatJsonString = JsonUtil.formatJsonString(paramString);
            log.info("chuli  shidu message after：" + formatJsonString);
            JSONObject msgJson = JSONObject.parseObject(formatJsonString);
            Long eqpId = msgJson.getLong("eqpId");
            String cpblName = msgJson.getString("cpblName");
            if (null != eqpId) {
                Map<String,Object> nodeDetailMap =  iTopographyNodeRepository.findNodeDetailBySourceId(msgJson.getString("sourceId"));
            	List<JSONObject> alarmAttributes = (List<JSONObject>) msgJson.get("capacityAttrList");

                synchronized (this) {
                    //拼接json存入动环详情表
                    Boolean alarmFlagBoolean = false;
                    for (JSONObject a : alarmAttributes) {
                        String atr = (String) a.get("name");
                        String res = (String) msgJson.get(atr);
                        if (res != null && (AlarmPointEnum.ABNORMAL.getName().equals(res)||AlarmPointEnum.ALARM.getName().equals(res))) {
                            alarmFlagBoolean = true;
                            log.info("Moving ring device[" + eqpId + "]" + atr + "abnormal");
                            break;
                        }
                    }

                    //组装动环信息JSON
                    List<Map<String, String>> DHlist = new ArrayList<>();
                    for (JSONObject a : alarmAttributes) {
                        String atr = (String) a.get("name");
                        String res = (String) msgJson.get(atr);
                        Map<String, String> map = new HashMap();
                        if (!"labelType".equals(atr)) {
                            map.put("name", a.get("displayName").toString());
                            map.put("value", msgJson.get(atr).toString());
                            DHlist.add(map);
                        }
                    }
                    if (nodeDetailMap.get("node_detail") != null) {
                        JSONArray nodeDetailArray = JSONArray.parseArray(nodeDetailMap.get("node_detail").toString());
                        for (int i = 0; i < DHlist.size(); i++) {
                            int count = 0;
                            for (int j = 0; j < nodeDetailArray.size(); j++) {
                                if (DHlist.get(i).get("name").equals(JSONObject.parseObject(nodeDetailArray.get(j).toString()).get("name"))) {
                                    nodeDetailArray.getJSONObject(j).put("value", DHlist.get(i).get("value"));
                                    count++;
                                }
                            }
                            if (count == 0) {
                                JSONObject detailObject = new JSONObject();
                                detailObject.put("name", DHlist.get(i).get("name"));
                                detailObject.put("value", DHlist.get(i).get("value"));
                                nodeDetailArray.add(detailObject);
                            }
                            TopographyNodeDetail topographyNodeDetail = iTopographyNodeDetailDao.findByNodeid(nodeDetailMap.get("id").toString());
                            topographyNodeDetail.setNodeDetail(nodeDetailArray.toJSONString());
                            iTopographyNodeDetailDao.save(topographyNodeDetail);
                        }
                    } else {
                        JSONArray nodeDetailArray = new JSONArray();
                        for (int i = 0; i < DHlist.size(); i++) {
                            JSONObject jsonObject = new JSONObject();
                            jsonObject.put("name", DHlist.get(i).get("name").toString());
                            jsonObject.put("value", DHlist.get(i).get("value").toString());
                            nodeDetailArray.add(jsonObject);
                        }
                        TopographyNodeDetail topographyNodeDetail = new TopographyNodeDetail();
                        topographyNodeDetail.setNodeDetail(JSON.toJSONString(nodeDetailArray));
                        topographyNodeDetail.setNodeid(nodeDetailMap.get("id").toString());
                        iTopographyNodeDetailDao.save(topographyNodeDetail);
                    }
                    //告警记录
                    String res = AlarmPointEnum.NORMAL.getName();
                    String alarmTypeCombination="";
                    for (JSONObject atrJson : alarmAttributes) {
                        String atrName = (String) atrJson.get("name");
                        if ("labelType".equals(atrName)) {
                            continue;
                        }
                        String alarmType = (String) atrJson.get("displayName");
                        if(AlarmPointEnum.ABNORMAL.getName().equals(msgJson.getString(atrName))||AlarmPointEnum.ALARM.getName().equals(msgJson.getString(atrName))){
                            alarmTypeCombination = alarmType + alarmTypeCombination;
                            res = AlarmPointEnum.ABNORMAL.getName();
                        }
                    }
                    Long cpblId = msgJson.getLong("cpblId");
                    String sourceId = msgJson.getString("sourceId");
                    String orgCode = msgJson.getString("orgCode");
                    String authOrg = msgJson.getString("authOrg");
                    String eqpName = msgJson.getString("eqpName");
                    String alramContent = msgJson.getString("alramContent");
                    String authOrgTemp = JsonUtil.arraysToString(JsonUtil.toStringArray(authOrg));
                    if (!ObjectUtils.isEmpty(authOrgTemp)) {
                        orgCode = authOrgTemp;

                    }
                    //添加告警记录
                    if(!"运行参数".equals(cpblName)||!"常电监测".equals(cpblName)||!"市电监测".equals(cpblName)||!"进行参数".equals(cpblName)){
                        addRecordAlarm(alarmTypeCombination, res, sourceId, orgCode, authOrg, eqpName,cpblName,alramContent);
                        List<AlarmPoint> pointList = alarmPointService.selectPointTypeAndPointId(AlarmPointTypeEnum.DH.getCode(), eqpId,cpblId);
                        List<Alarm> curAlarmList = alarmService.selectByQuerySourceId(sourceId);
                        if (!alarmFlagBoolean && pointList.size() > 0) {
                            alarmPointService.deleteItems(pointList);
                            log.info("Clear p_alarm_point  Alarm data ...success");
                            //设备所有的指标都不异常，才可以将state置为0
                            if(curAlarmList.size()==0){
                                iTopographyService.sendAbnormal(msgJson.getString("sourceId"), 0);
                            }
                        } else {
                            if (alarmFlagBoolean && pointList.size() == 0) {
//                                String alramContent = msgJson.getString("describe");
                                String eqpType = msgJson.getString("eqpType");
//                                String eqpName = msgJson.getString("eqpName");
                                String equipmentAttrs = msgJson.getString("equipmentAttrs");
//                                String sourceId = msgJson.getString("sourceId");
                                String authOrgs = getAuthOrg(msgJson.getString("authOrg"));

                                //String orgCode = msgJson.getString("orgCode");
                                AlarmPoint alarmPoint = new AlarmPoint();
                                alarmPoint.setCode(authOrgs);
                                alarmPoint.setSourceId(sourceId);
                                alarmPoint.setPointId(eqpId);
                                alarmPoint.setAuthOrg(authOrgs);
                                //    					alarmPoint.setPointType(eqpType);
                                alarmPoint.setPointType(AlarmPointTypeEnum.DH.getCode());
                                alarmPoint.setUpdateDate(new Date());
                                alarmPoint.setContent(eqpName+alarmTypeCombination);
                                alarmPoint.setPointAttrs(equipmentAttrs);
                                alarmPoint.setPointName(eqpName);
                                alarmPoint.setTarget(cpblId);
                                alarmPointService.saveItem(alarmPoint);
                                log.info("Save alarm data....success");

                                //推送消息巡检app
                                //String url = address + "api/msgSubscribe/alarmNotification";
                                try {
                                    Map<String, Object> map = new HashMap<String, Object>();
                                    map.put("code", msgJson.getString("orgCode"));
                                    map.put("body", "告警源名称：" + eqpName + TAB + "告警信息" + alramContent);
                                    String stringJson = JSONObject.toJSONString(map);
                                    //HttpUtil.PostJson(url, stringJson);
                                    CommonResponse result = patrolRemoteClient.sendAlarmNotification(stringJson);
                                    log.info("手机推送告警信息...success");
                                } catch (Exception e) {
                                    log.info("手机推送告警信息失败...fail" + e.getMessage());
                                }
                            }else {
                                if(pointList.size()>0){
                                    AlarmPoint alarmPoint =pointList.get(0);
                                    alarmPoint.setContent(eqpName+alarmTypeCombination);
                                    alarmPointService.save(alarmPoint);
                                }
                            }
                        }
                        if(alarmFlagBoolean){
                            iTopographyService.sendAbnormal(msgJson.getString("sourceId"), 1);
                        }
                        Sort sort = new Sort(Sort.Direction.DESC, "updateDate");
                        List<AlarmPoint> alarms = alarmPointService.findAll(sort);

                        WebsocketParam param = new WebsocketParam("alarmPoint", JSON.toJSONString(alarms));
                        Properties props = PropertiesLoaderUtils.loadAllProperties("application.properties");
                        String url = (String) props.get("params.remoteWebsocketUrl") + "/generic/sendMessage";
                        HttpUtil.PostJson(url, JSON.toJSONString(param));
                        log.info("send websocekt info：" + param);
                    }


                }
            }else {
				throw new YeeException("eqpId 不能为空！！！");
			}

        } catch (Exception e) {
            log.info("*********************error****************************");
            log.info(e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * 添加告警记录
     * @param alarmType 告警类型
     * @param alarmStatus 状态，正常 异常
     * @param sourceId
     * @param orgCode
     * @param authOrg
     * @param eqpName 设备名称
     * @param cpblName 报警类型
     */
    private void addRecordAlarm(String alarmType, String alarmStatus,String sourceId,String orgCode,String authOrg,String eqpName,String cpblName,String alramContent) {
    	//告警记录
        String queryColumn = sourceId + "_" + cpblName;
        //查出当前记录中的告警记录
        List<Alarm> curAlarmList = alarmService.findByQueryColumn(queryColumn);
        if (curAlarmList != null && curAlarmList.size() > 0) {
            Alarm alarm = curAlarmList.get(0);
            alarm.setOrgCode(orgCode);
            alarm.setUpdateDate(DateTimeKit.now());
            if (!ObjectUtils.isEmpty(alarm.getHappenDate())) {
                String subtract = DateTimeKit.getSubtract(alarm.getHappenDate());
                alarm.setContinueDate(subtract);
            }

            if (AlarmPointEnum.ALARM.getName().equals(alarmStatus)||AlarmPointEnum.ABNORMAL.getName().equals(alarmStatus)) {
                alarm.setCurrentState(ALARM_ABNORMAL_STATUS);
            } else {
                alarm.setClearDate(DateTimeKit.now());
                alarm.setClearPerson(CLEAR_PERSON);
                alarm.setCurrentState(ALARM_NORMAL_STATUS);
            }
            alarmMapper.updateAlarm(alarm);
        } else {//记录无告警则新增
        	 Alarm alarm = new Alarm();
             alarm.setSourceId(sourceId);
             alarm.setAlarmReason(cpblName);
             alarm.setAlarmSourceName(eqpName);
             alarm.setAlarmLevel(ALARM_DEVICE_MAJOR);
//             alarm.setAlarmSourceIp(sourceId);
             alarm.setAlarmSourceType(ALARM_DEVICE_TYPE);
             alarm.setClearDate("");
             alarm.setContinueDate("");
             alarm.setEnsureDate("");
             alarm.setEnsurePerson("");
             alarm.setOrgCode(orgCode);
             alarm.setHappenDate(DateTimeKit.now());
             alarm.setQueryColumn( sourceId + "_" + cpblName);
            List<Alarm> curAlarmRList = alarmService.findByQueryColumn(queryColumn);
             if (AlarmPointEnum.ALARM.getName().equals(alarmStatus)||AlarmPointEnum.ABNORMAL.getName().equals(alarmStatus)) {
                 alarm.setCurrentState(ALARM_ABNORMAL_STATUS);
                 iAlarmDao.save(alarm);
             } else {
                 alarm.setCurrentState(ALARM_NORMAL_STATUS);
             }
        }
        log.info("Save shidu alarm records ....success");
    }

    @Override
    public void onClose(int paramInt, String paramString, boolean paramBoolean) {
        log.info("close ...");
    }

    @Override
    public void onError(Exception e) {
        log.info("abnormal " + e);
    }
}
